#!/usr/data/php

<?php

use MomentCore\Core;

function __main__(){

	$akID	 = '?';       //设置你的阿里云key,用于调用api
	$akSec	 = '?'; //设置你的阿里云pw,用于调用api

	$updater = new MyDDNS($akID, $akSec);

	$updater->domain = 'imzlh.top';            //设置二级域名，例：ddns.stazhu.com 中的 stazhu.com
	$updater->prefix = 'cloud';                //设置二级域名的前缀，例：ddns.stazhu.com 中的 ddns
	$updater->ttl = 600;                       //设置TTL值，取决于你购买的云解析服务，TTL越低，DNS缓存更新越快，越佳，默认请设置为600，即10分钟。
	$updater->type = 'AAAA';                   //设置域名类型 IPv4 'A' IPv6 'AAAA'

	\Timer\interval(function() use (&$updater){
		$ip = yield Core::fetchText('http://6.ipw.cn',[],$H);
		if($ip == $updater -> ip){
			Core::log("\n{color_blue}I{/} 无需更新，你的IPV6是[{$updater->ip}]\n");
		}else{
			$updater -> ip = $ip;
			$result = yield $updater -> sendRequest();
			Core::log("{color_green}S{/} {color_gray}{$updater->prefix}.{$updater->domain}{/} REDIRECTED to {color_gray}{$updater->ip}{/}' \n");
		}
    },$updater->ttl / 3);

    Core::log('{color_green}I{/} AliDDNS by iz ready.');
} 

// Main Class
class MyDDNS
{
    public $accessKeyId;
    public $accessKeySecret;
    public $domain;
    public $ip;
    public $ttl;
    public $prefix;
    public $type;

	private $recID;

    function __construct($accessKeyId, $accessKeySecret) {
        $this->accessKeyId     = $accessKeyId;
        $this->accessKeySecret = $accessKeySecret;
    }

    public function sendRequest() {
        if(!is_string($this -> recID))
            $this -> recID = yield $this -> getRecordId();
        $queries = [
            'AccessKeyId' => $this->accessKeyId,
            'Action' => 'UpdateDomainRecord',
            'Format' => 'JSON',
            'RR' => $this->prefix,
            'RecordId' => $this->recID,
            'SignatureMethod' => 'HMAC-SHA1',
            'SignatureNonce' => rand(1000000000, PHP_INT_MAX),
            'SignatureVersion' => '1.0',
            'TTL' => $this->ttl,
            'Timestamp' => $this->getDate(),
            'Type' => $this->type,
            'Value' => $this->ip, 
            'Version' => '2015-01-09'
        ];

        return yield $this->doRequest($queries);
    }

    public function doRequest($queries) {
        $query = '';
        $i     = 0;

        foreach ($queries as $param => $query) {
            $query .= $i === 0 ? null : '&';
            $query .= "$param=$query";
            $i++;
        }

        $signature  = $this->getSignature($query);
        $requestUrl = "http://alidns.aliyuncs.com/?{$query}&Signature=" . urlencode($signature);			//如果服务器在非大陆地区不推荐用dns.aliyuncs.com，因为延时比较高，有的时候导致设置不成功，经过与阿里云沟通，他们提供了新地址。
        $response   = json_decode(yield Core::fetchText($requestUrl,[],$_),true);
        if($_['_status'] != 200)
            throw new Error("Request failed: {$response['Message']}");

        return $response;
    }

    public function GetRecordValue() {
        $queries = [
            'AccessKeyId' => $this->accessKeyId,
            'Action' => 'DescribeDomainRecords',
            'DomainName' => $this->domain,
            'Format' => 'JSON',
            'SignatureMethod' => 'HMAC-SHA1',
            'SignatureNonce' => rand(1000000000, PHP_INT_MAX),
            'SignatureVersion' => '1.0',
            'Timestamp' => $this->getDate(),
            'Version' => '2015-01-09'
        ];

        $response = $this->doRequest($queries);
        $recordList = $response['DomainRecords']['Record'];
        $prefix = null;

        foreach ($recordList as $key => $record) {
            if ($record['Type'] === $this->type && $this->prefix === $record['RR']) {
                $prefix = $record;
            }
        }

        if ($prefix === null) {
            die('prefix ' . $this->prefix . ' not found.');
        }
		
        //print_r($prefix);

        return $prefix['Value'];
    }

    public function getRecordId() {
        $queries = [
            'AccessKeyId' => $this->accessKeyId,
            'Action' => 'DescribeDomainRecords',
            'DomainName' => $this->domain,
            'Format' => 'JSON',
            'SignatureMethod' => 'HMAC-SHA1',
            'SignatureNonce' => rand(1000000000, PHP_INT_MAX),
            'SignatureVersion' => '1.0',
            'Timestamp' => $this->getDate(),
            'Version' => '2015-01-09'
        ];

        $response = yield $this->doRequest($queries);
        if(!is_array($response) || array_key_exists('Code',$response)){
		 	Core::log('{color_red}ERROR{/} Failed to send request');
			Core::dump($response);
			exit(1);
		}
		$recordList = $response['DomainRecords']['Record'];
        $prefix = null;

        foreach ($recordList as $key => $record) {
            if ($record['Type'] === $this->type && $this->prefix === $record['RR']) {
                $prefix = $record;
            }
        }

        if ($prefix === null) {
            Core::log('{color_red}E{/} AliDDNS fatal: prefix {color_yellow}' . $this->prefix . '{/} not found.');
            exit(1);
        }

        return $prefix['RecordId'];
    }

    public function getDate() {
        date_default_timezone_set('UTC');

        $date      = date('Y-m-d');
        $H         = date('H');
        $i         = date('i');
        $s         = date('s');

        return "{$date}T{$H}%3A{$i}%3A{$s}";
    }

    public function getSignature($query) {
        $HTTPMethod                  = 'GET';
        $slash                       = urlencode('/');
        $Encodedquery = urlencode($query);
        $StringToSign                = "{$HTTPMethod}&{$slash}&{$Encodedquery}";
        $StringToSign                = str_replace('%40', '%2540', $StringToSign);
        $StringToSign                = str_replace('%3A', '%253A', $StringToSign);
        $HMAC                        = hash_hmac('sha1', $StringToSign, "{$this->accessKeySecret}&", true);

        return base64_encode($HMAC);
    }
}
__main__();
?>
